@@ -1,18 +1,20 @@
-/**********************************************************************
- * Copyright (c) 2004 QNX Software Systems and others.
- * All rights reserved.   This program and the accompanying materials
- * are made available under the terms of the Common Public License v1.0
+/*******************************************************************************
+ * Copyright (c) 2004, 2005 QNX Software Systems and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
- * Contributors: 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
+ * Contributors:
  * QNX Software Systems - Initial API and implementation
-***********************************************************************/
+ *******************************************************************************/
 package org.eclipse.cdt.debug.internal.ui.views.disassembly;
 
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.Map;
+import org.eclipse.cdt.debug.core.CDebugCorePlugin;
+import org.eclipse.cdt.debug.core.ICDebugConstants;
 import org.eclipse.cdt.debug.core.model.ICStackFrame;
 import org.eclipse.cdt.debug.core.model.IDisassembly;
 import org.eclipse.cdt.debug.internal.ui.ICDebugHelpContextIds;
@@ -76,6 +78,7 @@ import org.eclipse.ui.ISelectionListener;
 import org.eclipse.ui.IViewPart;
 import org.eclipse.ui.IWorkbenchActionConstants;
 import org.eclipse.ui.IWorkbenchPart;
+import org.eclipse.ui.editors.text.EditorsUI;
 import org.eclipse.ui.internal.editors.text.EditorsPlugin;
 import org.eclipse.ui.texteditor.AbstractDecoratedTextEditorPreferenceConstants;
 import org.eclipse.ui.texteditor.AnnotationPreference;
@@ -281,7 +284,7 @@ public class DisassemblyView extends AbstractDebugEventHandlerView
 		viewer.configure( new DisassemblyViewerConfiguration() );
 		getSourceViewerDecorationSupport( viewer );
 		
-		EditorsPlugin.getDefault().getPreferenceStore().addPropertyChangeListener( this );
+		getEditorPreferenceStore().addPropertyChangeListener( this );
 		CDebugUIPlugin.getDefault().getPreferenceStore().addPropertyChangeListener( this );
 		getSite().getPage().addSelectionListener( IDebugUIConstants.ID_DEBUG_VIEW, this );
 		getSite().setSelectionProvider( viewer.getSelectionProvider() );
@@ -476,7 +479,7 @@ public class DisassemblyView extends AbstractDebugEventHandlerView
 		getSite().getPage().removeSelectionListener( IDebugUIConstants.ID_DEBUG_VIEW, this );
 		JFaceResources.getFontRegistry().removeListener( this );
 		JFaceResources.getColorRegistry().removeListener( this );
-		EditorsPlugin.getDefault().getPreferenceStore().removePropertyChangeListener( this );
+		getEditorPreferenceStore().removePropertyChangeListener( this );
 		CDebugUIPlugin.getDefault().getPreferenceStore().removePropertyChangeListener( this );
 
 		if ( fSourceViewerDecorationSupport != null ) {
@@ -669,7 +672,7 @@ public class DisassemblyView extends AbstractDebugEventHandlerView
 	}
 
 	private IPreferenceStore getEditorPreferenceStore() {
-		return EditorsPlugin.getDefault().getPreferenceStore();
+		return EditorsUI.getPreferenceStore();
 	}
 
 	public DisassemblyDocumentProvider getDocumentProvider() {
@@ -860,6 +863,11 @@ public class DisassemblyView extends AbstractDebugEventHandlerView
 		StyledText styledText = getSourceViewer().getTextWidget();
 		setTextContextMenu( manager.createContextMenu( styledText ) );
 		styledText.setMenu( getTextContextMenu() );
+
+		// register the context menu such that other plugins may contribute to it
+		if ( getSite() != null ) {
+			getSite().registerContextMenu( getViewContextMenuId(), manager, getSourceViewer() );
+		}
 	}
 
 	private void createRulerContextMenu() {
@@ -926,4 +934,24 @@ public class DisassemblyView extends AbstractDebugEventHandlerView
 			computeInput( input, (ICStackFrame)context, this );
 		}
 	}
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.ui.part.PageBookView#partActivated(org.eclipse.ui.IWorkbenchPart)
+	 */
+	public void partActivated( IWorkbenchPart part ) {
+		if ( this.equals( part ) ) {
+			CDebugCorePlugin.getDefault().getPluginPreferences().setValue( ICDebugConstants.PREF_INSTRUCTION_STEP_MODE_ON, true );
+		}
+		super.partActivated( part );
+	}
+
+	/* (non-Javadoc)
+	 * @see org.eclipse.ui.part.PageBookView#partDeactivated(org.eclipse.ui.IWorkbenchPart)
+	 */
+	public void partDeactivated( IWorkbenchPart part ) {
+		if ( this.equals( part ) ) {
+			CDebugCorePlugin.getDefault().getPluginPreferences().setValue( ICDebugConstants.PREF_INSTRUCTION_STEP_MODE_ON, false );
+		}
+		super.partDeactivated( part );
+	}
 }
